@@ -1,6 +1,6 @@
 /* Bitset vectors.
 
-   Copyright (C) 2002, 2004, 2009-2015, 2018 Free Software Foundation,
+   Copyright (C) 2002, 2004, 2009-2015, 2018-2019 Free Software Foundation,
    Inc.
 
    Contributed by Michael Hayes (m.hayes@elec.canterbury.ac.nz).
@@ -27,35 +27,38 @@ typedef bitset * bitsetv;
 
 /* Create a vector of N_VECS bitsets, each of N_BITS, and of
    type TYPE.  */
-extern bitsetv bitsetv_alloc (bitset_bindex, bitset_bindex, enum bitset_type);
+bitsetv bitsetv_alloc (bitset_bindex, bitset_bindex, enum bitset_type);
 
 /* Create a vector of N_VECS bitsets, each of N_BITS, and with
    attribute hints specified by ATTR.  */
-extern bitsetv bitsetv_create (bitset_bindex, bitset_bindex, unsigned);
+bitsetv bitsetv_create (bitset_bindex, bitset_bindex, unsigned);
 
 /* Free vector of bitsets.  */
-extern void bitsetv_free (bitsetv);
+void bitsetv_free (bitsetv);
 
 /* Zero vector of bitsets.  */
-extern void bitsetv_zero (bitsetv);
+void bitsetv_zero (bitsetv);
 
 /* Set vector of bitsets.  */
-extern void bitsetv_ones (bitsetv);
+void bitsetv_ones (bitsetv);
 
 /* Given a vector BSETV of N bitsets of size N, modify its contents to
    be the transitive closure of what was given.  */
-extern void bitsetv_transitive_closure (bitsetv);
+void bitsetv_transitive_closure (bitsetv);
 
 /* Given a vector BSETV of N bitsets of size N, modify its contents to
    be the reflexive transitive closure of what was given.  This is
    the same as transitive closure but with all bits on the diagonal
    of the bit matrix set.  */
-extern void bitsetv_reflexive_transitive_closure (bitsetv);
+void bitsetv_reflexive_transitive_closure (bitsetv);
 
 /* Dump vector of bitsets.  */
-extern void bitsetv_dump (FILE *, const char *, const char *, bitsetv);
+void bitsetv_dump (FILE *, const char *, const char *, bitsetv);
 
 /* Function to debug vector of bitsets from debugger.  */
-extern void debug_bitsetv (bitsetv);
+void debug_bitsetv (bitsetv);
+
+/* Dump vector of bitsets as a matrix.  */
+void bitsetv_matrix_dump (FILE *, const char *, bitsetv);
 
 #endif  /* _BITSETV_H  */
