@@ -1,7 +1,7 @@
 /* Compute lookahead criteria for bison,
 
    Copyright (C) 1984, 1986, 1989, 2000, 2002, 2004, 2006-2007,
-   2009-2015, 2018 Free Software Foundation, Inc.
+   2009-2015, 2018-2019 Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
