@@ -1,6 +1,7 @@
 /* Locations for Bison
 
-   Copyright (C) 2002, 2005-2015 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -37,7 +38,7 @@ static inline int
 add_column_width (int column, char const *buf, size_t bufsize)
 {
   size_t width;
-  unsigned int remaining_columns = INT_MAX - column;
+  unsigned remaining_columns = INT_MAX - column;
 
   if (buf)
     {
@@ -137,7 +138,7 @@ location_print (location loc, FILE *out)
 }
 
 
-/* Persistant data used by location_caret to avoid reopening and rereading the
+/* Persistent data used by location_caret to avoid reopening and rereading the
    same file all over for each error.  */
 struct caret_info
 {
@@ -208,13 +209,20 @@ location_caret (location loc, FILE *out)
           /* Print the carets (at least one), with the same indent as above.*/
           fprintf (out, " %*s", loc.start.column - 1, "");
           for (i = loc.start.column; i == loc.start.column || i < len; ++i)
-            putc ('^', out);
+            putc (i == loc.start.column ? '^' : '~', out);
           }
         putc ('\n', out);
       }
   }
 }
 
+bool
+location_empty (location loc)
+{
+  return !loc.start.file && !loc.start.line && !loc.start.column
+    && !loc.end.file && !loc.end.line && !loc.end.column;
+}
+
 void
 boundary_set_from_string (boundary *bound, char *loc_str)
 {
